﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.CombinedIntegerVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.LearningClassifierSystems {
  public class MaxValueActionSelector : ActionSelector {

    public ILookupParameter<ItemArray<DoubleValue>> ValueParameter {
      get { return (ILookupParameter<ItemArray<DoubleValue>>)Parameters["ValueParameter"]; }
    }

    [StorableConstructor]
    protected MaxValueActionSelector(bool deserializing) : base(deserializing) { }
    protected MaxValueActionSelector(MaxValueActionSelector original, Cloner cloner) : base(original, cloner) { }
    public MaxValueActionSelector()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("ValueParameter", "DoubleValue which will be summed up."));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MaxValueActionSelector(this, cloner);
    }

    protected override IMatching SelectAction(List<IScope> scopes) {
      Dictionary<IMatching, double> actionSet = new Dictionary<IMatching, double>();
      for (int i = 0; i < MatchParameter.ActualValue.Length; i++) {
        var action = MatchParameter.ActualValue[i].Action;
        if (actionSet.ContainsKey(action)) {
          actionSet[action] += ValueParameter.ActualValue[i].Value;
        } else {
          actionSet[action] = ValueParameter.ActualValue[i].Value;
        }
      }
      return actionSet.OrderByDescending(x => x.Value).First().Key;
    }
  }
}
